function [Qbc,Sbc,Qsr,Ssr]=sr_clustering_gc(A,noc,opts)

% The sr-clustering algortihm for graph clustering described in 
% M. Mrup and L.K.Hansen "An Exact Relaxation of the Hard Assignment
% Problem in Clustering", submitted to Journal of Machine Learning Research
% 2009.
%
% Optimizes the Modularity/Hamiltonian function
%   argmax_S 1/(2Z)*trace[S(A-P)S']  s.t. |S_i|_1=1, S_{d,i}>=0
%   where Z=sum(sum(A)), P is inherent null-hypothesis either given by
%   degree product, i.e. Modularity optimization or null-density rho,
%   i.e. Hamiltonian optimization.
%
% Usage: 
%       [Qbc,Sbc,Qsr,Ssr]=sr_clustering_gc(A,noc,opts)
%
% Input:
%   A       I x I  adjacency matrix (no self links)
%   noc     number of clusters
%   opts.
%           meth       'Modularity' (default) or 'Hamiltonian'
%           rho         null-hypothesis if meth='Hamiltonian', default
%                       rho=sum(sum(A))/I^2, i.e. average density
%           maxiter     maximum number of iterations
%           S           I x noc matrix of community identities
%           mu_s        initial stepsize (default 0.01)
%           verbose     1: display iteration results, 0: no display
%           delta       diagonal value in graph (default 0.1)
%
% Output:
%   Qbc     The final value of the objective 1/(2Z)*trace[S(A-P)S']
%   Sbc     Cluster indicator matrix of size d x I where d<=noc
%   Qsr     Value for potential non-binary configuration (if algorithm is stopped before convergence)
%   Ssr     Cluster indicator matrix of size d x I where d<=noc before
%           forcing solution to binary (if algorithm is stopped before convergence)

if nargin<3
    opts=struct;
end

% Initialize variables
[I,J]=size(A);
S=mgetopt(opts,'S',0.5+0.5*rand(noc,I));
maxiter=mgetopt(opts,'maxiter',500);
verbose=mgetopt(opts,'verbose',1);
S=S./repmat(sum(S),noc,1);
mu_s=mgetopt(opts,'mu_s',0.01);
meth=mgetopt(opts,'meth','Modularity');
delta=mgetopt(opts,'delta',1e-2);

% Generate null-hypothesis
if strcmp(meth,'Hamiltonian')
    k=ones(size(A,1),1);
    Z=sum(sum(A));
    m=1/mgetopt(opts,'rho',Z/(I*J)); 
else 
    k=sum(A,2);
    m=sum(k);
    Z=sum(k);
end
dQ=inf;
SA=S*A;
Sk=S*k;
Q=1/Z*(sum(sum(SA.*S))-sum(Sk.^2)/m+sum(sum(S.^2*(k.^2/m+delta))));

iter=0;
if verbose % Display algorithm
    disp(['Detecting communities using ' meth ]);
    dheader = sprintf('%12s | %12s | %12s | %12s | %12s','Iteration','Q','dQ/Q','step size','time');
    dline = sprintf('-------------+--------------+--------------+--------------+--------------');
    disp(dline);
    disp(dheader);
    disp(dline);
end

while dQ>abs(Q)*1e-10 && iter<maxiter
    iter=iter+1;
    tic;
    g=SA-(Sk)*k'/m+S.*repmat((k.^2)'/m+delta,noc,1);
    g=g-repmat(sum(g.*S),noc,1);
    Sold=S;
    stop=0;
    while ~stop % perform projected L1 constrained gradient ascent
        S=Sold+mu_s*g/max(abs(g(:)));
        S(S<0)=0;
        S=S./repmat(sum(S),noc,1);
        SA=S*A; 
        Sk=S*k;
        dkk=sum(sum(S.^2*(k.^2/m+delta)));
        Qnew=1/Z*(sum(sum(SA.*S))-sum(Sk.^2)/m+dkk);
        dQ=Qnew-Q;
        if dQ>=0 | mu_s<1e-12
            mu_s=mu_s*1.2;
            stop=1;
            Q=Qnew;
        else
            if verbose
                disp('Checking stationarity')
            end
            mu_s=mu_s/10;
        end
    end
    %if mod(iter,10)==0
    %    figure(1); imagesc(S); pause(0.1);
    %end
    if rem(iter,5)==0 && verbose
        disp(sprintf('%12.0f | %12.4e | %12.4e | %12.4e | %12.4f ',iter,Q-dkk/Z,dQ/Q,mu_s,toc));
    end
end

if verbose
    disp(sprintf('%12.0f | %12.4e | %12.4e | %12.4e | %12.4f ',iter,Q-dkk/Z,dQ/Q,mu_s,toc));
end
% Remove empty cluster
[val,l]=sort(sum(S,2),'descend');
Ssr=S(l,:);
Ssr(val<=eps,:)=[];
Qsr=1/Z*(sum(sum(SA.*S))-sum(Sk.^2)/m);

% Force binary
[val,ind]=max(Ssr);
Sbc=sparse(ind,1:J,ones(size(val)),max(ind),J);
Sk=(Sbc*k);
SA=Sbc*A; 
Qbc=full(1/Z*(sum(sum(SA.*Sbc))-sum(Sk.^2)/m));
   
% -------------------------------------------------------------------------
% Parser for optional arguments
function var = mgetopt(opts, varname, default, varargin)
if isfield(opts, varname)
    var = getfield(opts, varname); 
else
    var = default;
end
for narg = 1:2:nargin-4
    cmd = varargin{narg};
    arg = varargin{narg+1};
    switch cmd
        case 'instrset',
            if ~any(strcmp(arg, var))
                fprintf(['Wrong argument %s = ''%s'' - ', ...
                    'Using default : %s = ''%s''\n'], ...
                    varname, var, varname, default);
                var = default;
            end
        otherwise,
            error('Wrong option: %s.', cmd);
    end
end

